package robotsfx.com.example.wimaster_test_android4;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

public class MainActivity extends Activity {
	//メンバ
	private final static String TAG="WiMaster_test4";
	private final static String IP="1.2.3.4"; 	//wimaster IPアドレス(固定)
	private final static int PORT=2000;			//wimaster PORT番号(固定)
	private final static int WifiComWhat = 1;	//ハンドラの識別用定数
	private Socket		 socket; 				//ソケット
	private InputStream  in;					//入力ストリーム
	private OutputStream out;					//出力ストリーム
	private static TextView sensorText;			//センサー値表示用テキスト
	private Button onButton;					//LED ON 用ボタン
	private Button oｆｆButton;					//LED ON 用ボタン
	
	/////////////////////////////////////////////////
	//	起動時処理
	/////////////////////////////////////////////////
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		//---UI---
		sensorText = (TextView)findViewById(R.id.sensor_text);
		onButton = (Button)findViewById(R.id.on_button);
		onButton.setOnClickListener(new onButtonClick());
		oｆｆButton = (Button)findViewById(R.id.off_button);
		oｆｆButton.setOnClickListener(new onButtonClick());
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	////////////////////////////////////////////////
    //	onResume
	////////////////////////////////////////////////
    @Override
    public void onResume(){
    	super.onResume();	
    	//スレッド生成   	
    	Thread thread = new Thread(){
    		public void run(){
    			try{
    				connect(IP, PORT);		//resume時にソケットを開く
    			}catch(Exception e){
    				e.printStackTrace();
    			}
    		}
    	};
    	thread.start();
    } 
    
	////////////////////////////////////////////////
    //	onPause
	////////////////////////////////////////////////
    @Override
    public void onPause(){
    	//pause時にソケットを閉じる
    	try{
    		socket.close();
    		socket=null;
    	}catch(Exception e){	
    	}
    	super.onPause();
    }
    
	/////////////////////////////////////////////////
    //	接続
	/////////////////////////////////////////////////
    private void connect(String ip,int port) {
        Log.v(TAG,"connect start");
        int size;
        byte[] buffer=new byte[1024];
        try {
            //ソケット接続
            socket=new Socket(ip,port);            
            if( socket.isConnected() && socket != null)
            {
                in =socket.getInputStream();
                out=socket.getOutputStream();
                Log.v("connect","接続成功");
            }
            else
            {
                Log.v("connect","接続失敗");
            }
            //受信ループ
            while( socket.isConnected() && socket != null){
            	size = in.read(buffer);
            	if(size<=0) continue;
    			//データをハンドラに渡す
    			byte[] data = new byte[size];
    			System.arraycopy(buffer, 0, data, 0, size);
    			Log.d(TAG,"data length="+data.length);
				handler.obtainMessage(WifiComWhat, data).sendToTarget();
            }
            
        } catch (Exception e) {
            Log.v("connect", e.toString() );
        }
    }
    
	/////////////////////////////////////////////////
	//	ボタン処理
	/////////////////////////////////////////////////
	class onButtonClick implements OnClickListener{
		@Override
		public void onClick(View v){
			switch (v.getId()) {
			case R.id.on_button:		//on
				sendLedOnCommand();
				break;
			case R.id.off_button:		//off
				sendLedOffCommand();
				break;
			
			}
		}
	}
	
	/////////////////////////////////////////////////
	//	LED操作命令送信
	/////////////////////////////////////////////////
	private void sendLedOnCommand(){
		try{
			byte[] data = new byte[1];
			data[0] = (byte)1;		//on command
			sendData(data);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	private void sendLedOffCommand(){
		try{
			byte[] data = new byte[1];
			data[0] = (byte)0;		//off command
			sendData(data);
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	////////////////////////////////////////////////
    //	データ送信
	////////////////////////////////////////////////	
    private void sendData(byte[] data){
    	try{
			//データの送信
			if(socket!=null && socket.isConnected()){
				out.write(data);
				out.flush();
			}
		}catch(Exception e){
			Log.d(TAG,"送信！！失敗しました");
			e.printStackTrace();
		}
    }
    
    ////////////////////////////////////////////////
    //	イベントハンドラ(受信)
	////////////////////////////////////////////////	
    private static Handler handler = new Handler() {
    	@Override
    	public void handleMessage(Message msg) {
			switch(msg.what)
			{	
			case WifiComWhat:
				byte[] data = (byte[])msg.obj;
				//受信データdebug表示
				String str = "";
				for(int i=0; i<data.length; i++){
					str += String.valueOf(data[i]) + ",";
				}
				Log.d(TAG,"data="+str);
				chkRcvData(data);
				break;
			default:
				break;
			}	//switch
    	} //handleMessage
    }; //handler
    
    ////////////////////////////////////////////////
    //	受信データの表示
	////////////////////////////////////////////////
    private static void chkRcvData(byte[] data){
    		String str = String.valueOf(convByteToInt(data[0]));	//*1
    		sensorText.setText(str);
    }
    //*1JAVAでは8bit値は符号無しのみなので、127以上はマイナスの値に表示されてしまう。よってこれを符号無し32bitのintへ変換している。
    //センサーのポート番号は＃１～１６、byte配列は０～１５なので＃３の値はsensorValue[2]になる。
    
    ///////////////////////////////////////////////
    //	byteを符号無しintへ変換
	////////////////////////////////////////////////
    private static int convByteToInt(byte b){
		int i = (int)b;
		i &= 0x000000FF;
		return i;
	}
    
}
